/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.  Users and possessors of this source code
 * are hereby granted a nonexclusive, royalty-free license to use this code
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

/* Matrix multiplication: C = A * B.
 * Host code.
 *
 * This sample implements matrix multiplication and is exactly the same as
 * Chapter 7 of the programming guide.
 * It has been written for clarity of exposition to illustrate various CUDA
 * programming principles, not with the goal of providing the most
 * performant generic kernel for matrix multiplication.
 *
 * CUBLAS provides high-performance matrix multiplication.
 */

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include <cutil.h>

// includes, kernels
#include <matrixMul_kernel.cu>

////////////////////////////////////////////////////////////////////////////////
// declaration, forward
int runTest(); //int argc, char** argv);
void randomInit(float*, int);
void printDiff(float*, float*, int, int);

extern "C"
void computeGold(float*, const float*, const float*, unsigned int, unsigned int, unsigned int);

////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////

/*

i have added MatrixMul sample from Nvidia cuda program to a simple existing project using mfc,
the debug version can link and run.
in order to achieve this, i've modified the file .proj and .sln

but if you compile with the emudebug version, visual studio compiler tell :
1>libcmtd.lib(crt0.obj) : error LNK2019: unresolved external symbol _main referenced in function ___tmainCRTStartup 
you can compile and link the program if you uncomment the line below, in order to have the main() function

BUT when you use MFC, you musn't have a main() function, "main()" is executed somewhere else by MFC

If you have  a main() function, you will execute just the part below in console mode, and don't execute the MFC application.

for linking the program you must set IgnoreDefaultLibraryNames : "LIBCMTD.lib"


It would be very useful if Nvidia, or developer, provide an utility that modify the   .proj file, and the .sln   file
of an existing project, without having to modify all the properties in visual studio by hand.

You can open those files (text file), by right-clic, and "open with",   "notepad"

_________________________________________________________________________
test.proj file :
__________________
in part Name="VCCLCompilerTool",  add :
	AdditionalIncludeDirectories="$(CUDA_INC_PATH);C:\Program Files\NVIDIA Corporation\NVIDIA CUDA SDK\common\inc"

in part	Name="VCLinkerTool", add :
				AdditionalDependencies="cudart.lib cutil32D.lib"
				AdditionalLibraryDirectories="$(CUDA_LIB_PATH);C:\Program Files\NVIDIA Corporation\NVIDIA CUDA SDK\common\lib"
				IgnoreDefaultLibraryNames="LIBCMTD.lib"

in part :
					<Tool
						Name="VCCustomBuildTool"
	for release and debug mode :
						CommandLine="&quot;$(CUDA_BIN_PATH)\nvcc.exe&quot; -ccbin &quot;$(VCInstallDir)bin&quot; -c -D_DEBUG -DWIN32 -D_CONSOLE -D_MBCS -Xcompiler /EHsc,/W3,/nologo,/Wp64,/Od,/Zi,/RTC1,/MTd -I&quot;$(CUDA_INC_PATH)&quot; -I./ -I&quot;C:\Program Files\NVIDIA Corporation\NVIDIA CUDA SDK\common\inc&quot; -o $(ConfigurationName)\matrixMul.obj matrixMul.cu&#x0D;&#x0A;"

    for emudebug and emurelease, just add :  -deviceemu     
	                   CommandLine="&quot;$(CUDA_BIN_PATH)\nvcc.exe&quot; -ccbin &quot;$(VCInstallDir)bin&quot; -deviceemu -c -D_DEBUG .....

________________________________________________________________________
test.sln file :
__________________

add :

		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Debug|x64.ActiveCfg = EmuDebug|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Debug|x64.Build.0 = EmuDebug|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuDebug|Win32.ActiveCfg = EmuDebug|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuDebug|Win32.Build.0 = EmuDebug|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuDebug|x64.ActiveCfg = EmuDebug|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuDebug|x64.Build.0 = EmuDebug|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuRelease|Win32.ActiveCfg = EmuRelease|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuRelease|Win32.Build.0 = EmuRelease|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuRelease|x64.ActiveCfg = EmuRelease|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.EmuRelease|x64.Build.0 = EmuRelease|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Release|x64.ActiveCfg = EmuRelease|x64
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Release|x64.Build.0 = EmuRelease|x64

the key		84731EDA-1B7B-4C24-92D4-ED24A74D9BE6     depend of your project
it must the same as the one is the line, of your existing MFC project
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Debug|Win32.ActiveCfg = Debug|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Debug|Win32.Build.0 = Debug|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Release|Win32.ActiveCfg = Release|Win32
		{84731EDA-1B7B-4C24-92D4-ED24A74D9BE6}.Release|Win32.Build.0 = Release|Win32


i've not found a solution to tell the compiler what is the path, in order to load cutil32.lib  or cutil32D.lib
so, i've copied those files in the project.

compiled with VISUAL STUDIO 2005, and CUDA 2.0 beta 2b
If someone has a solution, tell us...

*/

/*
int
main(int argc, char** argv)
{
    runTest(); //argc, argv);

    CUT_EXIT(argc, argv);
}
*/
////////////////////////////////////////////////////////////////////////////////
//! Run a simple test for CUDA
////////////////////////////////////////////////////////////////////////////////
int
runTest() //int argc, char** argv)
{
    //CUT_DEVICE_INIT(argc, argv);

    // set seed for rand()
    srand(2006);

    // allocate host memory for matrices A and B
    unsigned int size_A = WA * HA;
    unsigned int mem_size_A = sizeof(float) * size_A;
    float* h_A = (float*) malloc(mem_size_A);
    unsigned int size_B = WB * HB;
    unsigned int mem_size_B = sizeof(float) * size_B;
    float* h_B = (float*) malloc(mem_size_B);

    // initialize host memory
    randomInit(h_A, size_A);
    randomInit(h_B, size_B);

    // allocate device memory
    float* d_A;
    CUDA_SAFE_CALL(cudaMalloc((void**) &d_A, mem_size_A));
    float* d_B;
    CUDA_SAFE_CALL(cudaMalloc((void**) &d_B, mem_size_B));

    // copy host memory to device
    CUDA_SAFE_CALL(cudaMemcpy(d_A, h_A, mem_size_A,
                              cudaMemcpyHostToDevice) );
    CUDA_SAFE_CALL(cudaMemcpy(d_B, h_B, mem_size_B,
                              cudaMemcpyHostToDevice) );

    // allocate device memory for result
    unsigned int size_C = WC * HC;
    unsigned int mem_size_C = sizeof(float) * size_C;
    float* d_C;
    CUDA_SAFE_CALL(cudaMalloc((void**) &d_C, mem_size_C));

    // allocate host memory for the result
    float* h_C = (float*) malloc(mem_size_C);
    
    // create and start timer
    unsigned int timer = 0;
    CUT_SAFE_CALL(cutCreateTimer(&timer));
    CUT_SAFE_CALL(cutStartTimer(timer));

    // setup execution parameters
    dim3 threads(BLOCK_SIZE, BLOCK_SIZE);
    dim3 grid(WC / threads.x, HC / threads.y);

    // execute the kernel
    matrixMul<<< grid, threads >>>(d_C, d_A, d_B, WA, WB);

    // check if kernel execution generated and error
    CUT_CHECK_ERROR("Kernel execution failed");

    // copy result from device to host
    CUDA_SAFE_CALL(cudaMemcpy(h_C, d_C, mem_size_C,
                              cudaMemcpyDeviceToHost) );

    // stop and destroy timer
    CUT_SAFE_CALL(cutStopTimer(timer));
    printf("Processing time: %f (ms) \n", cutGetTimerValue(timer));
    CUT_SAFE_CALL(cutDeleteTimer(timer));

    // compute reference solution
    float* reference = (float*) malloc(mem_size_C);
    computeGold(reference, h_A, h_B, HA, WA, WB);

    // check result
    CUTBoolean res = cutCompareL2fe(reference, h_C, size_C, 1e-6f);
    printf("Test %s \n", (1 == res) ? "PASSED" : "FAILED");
    if (res!=1) printDiff(reference, h_C, WC, HC);

    // clean up memory
    free(h_A);
    free(h_B);
    free(h_C);
    free(reference);
    CUDA_SAFE_CALL(cudaFree(d_A));
    CUDA_SAFE_CALL(cudaFree(d_B));
    CUDA_SAFE_CALL(cudaFree(d_C));
	return(res);
}

// Allocates a matrix with random float entries.
void randomInit(float* data, int size)
{
    for (int i = 0; i < size; ++i)
        data[i] = rand() / (float)RAND_MAX;
}

void printDiff(float *data1, float *data2, int width, int height)
{
  int i,j,k;
  int error_count=0;
  for (j=0; j<height; j++) {
    for (i=0; i<width; i++) {
      k = j*width+i;
      if (data1[k] != data2[k]) {
         printf("diff(%d,%d) CPU=%4.4f, GPU=%4.4f n", i,j, data1[k], data2[k]);
         error_count++;
      }
    }
  }
  printf(" nTotal Errors = %d n", error_count);
}

